@@ -4,12 +4,14 @@
 // Based on biosnoop(8) from BCC by Brendan Gregg.
 // 29-Jun-2020   Wenbo Zhang   Created this.
 #include <argp.h>
+#include <signal.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <time.h>
 #include <bpf/libbpf.h>
 #include <sys/resource.h>
 #include <bpf/bpf.h>
+#include <fcntl.h>
 #include "blk_types.h"
 #include "biosnoop.h"
 #include "biosnoop.skel.h"
@@ -18,12 +20,16 @@
 #define PERF_BUFFER_PAGES	16
 #define PERF_POLL_TIMEOUT_MS	100
 
+static volatile sig_atomic_t exiting = 0;
+
 static struct env {
 	char *disk;
 	int duration;
 	bool timestamp;
 	bool queued;
 	bool verbose;
+	char *cgroupspath;
+	bool cg;
 } env = {};
 
 static volatile __u64 start_ts;
@@ -34,18 +40,20 @@ const char *argp_program_bug_address =
 const char argp_program_doc[] =
 "Trace block I/O.\n"
 "\n"
-"USAGE: biosnoop [--help] [-d DISK] [-Q]\n"
+"USAGE: biosnoop [--help] [-d DISK] [-c CG] [-Q]\n"
 "\n"
 "EXAMPLES:\n"
 "    biosnoop              # trace all block I/O\n"
 "    biosnoop -Q           # include OS queued time in I/O time\n"
 "    biosnoop 10           # trace for 10 seconds only\n"
-"    biosnoop -d sdc       # trace sdc only\n";
+"    biosnoop -d sdc       # trace sdc only\n"
+"    biosnoop -c CG        # Trace process under cgroupsPath CG\n";
 
 static const struct argp_option opts[] = {
 	{ "queued", 'Q', NULL, 0, "Include OS queued time in I/O time" },
 	{ "disk",  'd', "DISK",  0, "Trace this disk only" },
 	{ "verbose", 'v', NULL, 0, "Verbose debug output" },
+	{ "cgroup", 'c', "/sys/fs/cgroup/unified/CG", 0, "Trace process in cgroup path"},
 	{ NULL, 'h', NULL, OPTION_HIDDEN, "Show the full help" },
 	{},
 };
@@ -64,6 +72,10 @@ static error_t parse_arg(int key, char *arg, struct argp_state *state)
 	case 'Q':
 		env.queued = true;
 		break;
+	case 'c':
+		env.cg = true;
+		env.cgroupspath = arg;
+		break;
 	case 'd':
 		env.disk = arg;
 		if (strlen(arg) + 1 > DISK_NAME_LEN) {
@@ -90,14 +102,18 @@ static error_t parse_arg(int key, char *arg, struct argp_state *state)
 	return 0;
 }
 
-int libbpf_print_fn(enum libbpf_print_level level,
-		const char *format, va_list args)
+static int libbpf_print_fn(enum libbpf_print_level level, const char *format, va_list args)
 {
 	if (level == LIBBPF_DEBUG && !env.verbose)
 		return 0;
 	return vfprintf(stderr, format, args);
 }
 
+static void sig_int(int signo)
+{
+	exiting = 1;
+}
+
 static void blk_fill_rwbs(char *rwbs, unsigned int op)
 {
 	int i = 0;
@@ -151,7 +167,7 @@ void handle_event(void *ctx, int cpu, void *data, __u32 data_sz)
 		start_ts = e->ts;
 	blk_fill_rwbs(rwbs, e->cmd_flags);
 	partition = partitions__get_by_dev(partitions, e->dev);
-	printf("%-11.6f %-14.14s %-6d %-7s %-4s %-10lld %-7d ",
+	printf("%-11.6f %-14.14s %-7d %-7s %-4s %-10lld %-7d ",
 		(e->ts - start_ts) / 1000000000.0,
 		e->comm, e->pid, partition ? partition->name : "Unknown", rwbs,
 		e->sector, e->len);
@@ -174,25 +190,21 @@ int main(int argc, char **argv)
 		.parser = parse_arg,
 		.doc = argp_program_doc,
 	};
-	struct perf_buffer_opts pb_opts;
 	struct perf_buffer *pb = NULL;
 	struct ksyms *ksyms = NULL;
 	struct biosnoop_bpf *obj;
 	__u64 time_end = 0;
 	int err;
+	int idx, cg_map_fd;
+	int cgfd = -1;
 
 	err = argp_parse(&argp, argc, argv, 0, NULL, NULL);
 	if (err)
 		return err;
 
+	libbpf_set_strict_mode(LIBBPF_STRICT_ALL);
 	libbpf_set_print(libbpf_print_fn);
 
-	err = bump_memlock_rlimit();
-	if (err) {
-		fprintf(stderr, "failed to increase rlimit: %d\n", err);
-		return 1;
-	}
-
 	obj = biosnoop_bpf__open();
 	if (!obj) {
 		fprintf(stderr, "failed to open BPF object\n");
@@ -212,8 +224,18 @@ int main(int argc, char **argv)
 			fprintf(stderr, "invaild partition name: not exist\n");
 			goto cleanup;
 		}
+		obj->rodata->filter_dev = true;
+		obj->rodata->targ_dev = partition->dev;
 	}
 	obj->rodata->targ_queued = env.queued;
+	obj->rodata->filter_cg = env.cg;
+
+	if (fentry_can_attach("blk_account_io_start", NULL))
+		bpf_program__set_attach_target(obj->progs.blk_account_io_start, 0,
+					       "blk_account_io_start");
+	else
+		bpf_program__set_attach_target(obj->progs.blk_account_io_start, 0,
+					       "__blk_account_io_start");
 
 	err = biosnoop_bpf__load(obj);
 	if (err) {
@@ -221,69 +243,75 @@ int main(int argc, char **argv)
 		goto cleanup;
 	}
 
-	obj->links.blk_account_io_start =
-		bpf_program__attach(obj->progs.blk_account_io_start);
-	err = libbpf_get_error(obj->links.blk_account_io_start);
-	if (err) {
+	/* update cgroup path fd to map */
+	if (env.cg) {
+		idx = 0;
+		cg_map_fd = bpf_map__fd(obj->maps.cgroup_map);
+		cgfd = open(env.cgroupspath, O_RDONLY);
+		if (cgfd < 0) {
+			fprintf(stderr, "Failed opening Cgroup path: %s\n", env.cgroupspath);
+			goto cleanup;
+		}
+		if (bpf_map_update_elem(cg_map_fd, &idx, &cgfd, BPF_ANY)) {
+			fprintf(stderr, "Failed adding target cgroup to map\n");
+			goto cleanup;
+		}
+	}
+
+	obj->links.blk_account_io_start = bpf_program__attach(obj->progs.blk_account_io_start);
+	if (!obj->links.blk_account_io_start) {
+		err = -errno;
 		fprintf(stderr, "failed to attach blk_account_io_start: %s\n",
-			strerror(err));
+			strerror(-err));
 		goto cleanup;
 	}
 	ksyms = ksyms__load();
 	if (!ksyms) {
+		err = -ENOMEM;
 		fprintf(stderr, "failed to load kallsyms\n");
 		goto cleanup;
 	}
 	if (ksyms__get_symbol(ksyms, "blk_account_io_merge_bio")) {
 		obj->links.blk_account_io_merge_bio =
 			bpf_program__attach(obj->progs.blk_account_io_merge_bio);
-		err = libbpf_get_error(obj->links.blk_account_io_merge_bio);
-		if (err) {
-			fprintf(stderr, "failed to attach "
-				"blk_account_io_merge_bio: %s\n",
-				strerror(err));
+		if (!obj->links.blk_account_io_merge_bio) {
+			err = -errno;
+			fprintf(stderr, "failed to attach blk_account_io_merge_bio: %s\n",
+				strerror(-err));
 			goto cleanup;
 		}
 	}
 	if (env.queued) {
 		obj->links.block_rq_insert =
 			bpf_program__attach(obj->progs.block_rq_insert);
-		err = libbpf_get_error(obj->links.block_rq_insert);
-		if (err) {
-			fprintf(stderr, "failed to attach block_rq_insert: %s\n",
-				strerror(err));
+		if (!obj->links.block_rq_insert) {
+			err = -errno;
+			fprintf(stderr, "failed to attach block_rq_insert: %s\n", strerror(-err));
 			goto cleanup;
 		}
 	}
-	obj->links.block_rq_issue =
-		bpf_program__attach(obj->progs.block_rq_issue);
-	err = libbpf_get_error(obj->links.block_rq_issue);
-	if (err) {
-		fprintf(stderr, "failed to attach block_rq_issue: %s\n",
-			strerror(err));
+	obj->links.block_rq_issue = bpf_program__attach(obj->progs.block_rq_issue);
+	if (!obj->links.block_rq_issue) {
+		err = -errno;
+		fprintf(stderr, "failed to attach block_rq_issue: %s\n", strerror(-err));
 		goto cleanup;
 	}
-	obj->links.block_rq_complete =
-		bpf_program__attach(obj->progs.block_rq_complete);
-	err = libbpf_get_error(obj->links.block_rq_complete);
-	if (err) {
-		fprintf(stderr, "failed to attach block_rq_complete: %s\n",
-			strerror(err));
+	obj->links.block_rq_complete = bpf_program__attach(obj->progs.block_rq_complete);
+	if (!obj->links.block_rq_complete) {
+		err = -errno;
+		fprintf(stderr, "failed to attach block_rq_complete: %s\n", strerror(-err));
 		goto cleanup;
 	}
 
-	pb_opts.sample_cb = handle_event;
-	pb_opts.lost_cb = handle_lost_events;
 	pb = perf_buffer__new(bpf_map__fd(obj->maps.events), PERF_BUFFER_PAGES,
-			&pb_opts);
-	err = libbpf_get_error(pb);
-	if (err) {
-		pb = NULL;
+			      handle_event, handle_lost_events, NULL, NULL);
+	if (!pb) {
+		err = -errno;
 		fprintf(stderr, "failed to open perf buffer: %d\n", err);
 		goto cleanup;
 	}
 
-	printf("%-11s %-14s %-6s %-7s %-4s %-10s %-7s ",
+	printf("%-11s %-14s %-7s %-7s %-4s %-10s %-7s ",
 		"TIME(s)", "COMM", "PID", "DISK", "T", "SECTOR", "BYTES");
 	if (env.queued)
 		printf("%7s ", "QUE(ms)");
@@ -293,19 +321,32 @@ int main(int argc, char **argv)
 	if (env.duration)
 		time_end = get_ktime_ns() + env.duration * NSEC_PER_SEC;
 
+	if (signal(SIGINT, sig_int) == SIG_ERR) {
+		fprintf(stderr, "can't set signal handler: %s\n", strerror(errno));
+		err = 1;
+		goto cleanup;
+	}
+
 	/* main: poll */
-	while (1) {
-		if ((err = perf_buffer__poll(pb, PERF_POLL_TIMEOUT_MS)) < 0)
+	while (!exiting) {
+		err = perf_buffer__poll(pb, PERF_POLL_TIMEOUT_MS);
+		if (err < 0 && err != -EINTR) {
+			fprintf(stderr, "error polling perf buffer: %s\n", strerror(-err));
+			goto cleanup;
+		}
+		/* reset err to return 0 if exiting */
+		err = 0;
+		if (env.duration && get_ktime_ns() > time_end)
 			break;
-		if (env.duration && get_ktime_ns() > time_end)
-			goto cleanup;
 	}
-	printf("error polling perf buffer: %d\n", err);
 
 cleanup:
+	perf_buffer__free(pb);
 	biosnoop_bpf__destroy(obj);
 	ksyms__free(ksyms);
 	partitions__free(partitions);
+	if (cgfd > 0)
+		close(cgfd);
 
 	return err != 0;
 }
