@@ -6,12 +6,14 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <time.h>
+#include <unistd.h>
 #include <argp.h>
 #include <bpf/bpf.h>
 #include "syscount.h"
 #include "syscount.skel.h"
 #include "errno_helpers.h"
 #include "syscall_helpers.h"
+#include "btf_helpers.h"
 #include "trace_helpers.h"
 
 /* This structure extends data_t by adding a key item which should be sorted
@@ -38,7 +40,8 @@ static const char argp_program_doc[] =
 "    syscount -L              # measure and sort output by latency\n"
 "    syscount -P              # group statistics by pid, not by syscall\n"
 "    syscount -x -i 5         # count only failed syscalls\n"
-"    syscount -e ENOENT -i 5  # count only syscalls failed with a given errno"
+"    syscount -e ENOENT -i 5  # count only syscalls failed with a given errno\n"
+"    syscount -c CG           # Trace process under cgroupsPath CG\n";
 ;
 
 static const struct argp_option opts[] = {
@@ -48,6 +51,7 @@ static const struct argp_option opts[] = {
 				" (seconds), 0 for infinite wait (default)" },
 	{ "duration", 'd', "DURATION", 0, "Total tracing duration (seconds)" },
 	{ "top", 'T', "TOP", 0, "Print only the top syscalls (default 10)" },
+	{ "cgroup", 'c', "/sys/fs/cgroup/unified/<CG>", 0, "Trace process in cgroup path"},
 	{ "failures", 'x', NULL, 0, "Trace only failed syscalls" },
 	{ "latency", 'L', NULL, 0, "Collect syscall latency" },
 	{ "milliseconds", 'm', NULL, 0, "Display latency in milliseconds"
@@ -72,6 +76,8 @@ static struct env {
 	int duration;
 	int top;
 	pid_t pid;
+	char *cgroupspath;
+	bool cg;
 } env = {
 	.top = 10,
 };
@@ -94,8 +100,7 @@ static int get_int(const char *arg, int *ret, int min, int max)
 	return 0;
 }
 
-static int libbpf_print_fn(enum libbpf_print_level level,
-			   const char *format, va_list args)
+static int libbpf_print_fn(enum libbpf_print_level level, const char *format, va_list args)
 {
 	if (level == LIBBPF_DEBUG && !env.verbose)
 		return 0;
@@ -335,6 +340,10 @@ static error_t parse_arg(int key, char *arg, struct argp_state *state)
 			argp_usage(state);
 		}
 		break;
+	case 'c':
+		env.cgroupspath = arg;
+		env.cg = true;
+		break;
 	case 'e':
 		err = get_int(arg, &number, 1, INT_MAX);
 		if (err) {
@@ -367,6 +376,7 @@ void sig_int(int signo)
 
 int main(int argc, char **argv)
 {
+	LIBBPF_OPTS(bpf_object_open_opts, open_opts);
 	void (*print)(struct data_ext_t *, size_t);
 	int (*compar)(const void *, const void *);
 	static const struct argp argp = {
@@ -379,6 +389,8 @@ int main(int argc, char **argv)
 	int seconds = 0;
 	__u32 count;
 	int err;
+	int idx, cg_map_fd;
+	int cgfd = -1;
 
 	init_syscall_names();
 
@@ -391,15 +403,16 @@ int main(int argc, char **argv)
 		goto free_names;
 	}
 
+	libbpf_set_strict_mode(LIBBPF_STRICT_ALL);
 	libbpf_set_print(libbpf_print_fn);
 
-	err = bump_memlock_rlimit();
+	err = ensure_core_btf(&open_opts);
 	if (err) {
-		warn("failed to increase rlimit: %s\n", strerror(errno));
-		goto free_names;
+		fprintf(stderr, "failed to fetch necessary BTF for CO-RE: %s\n", strerror(-err));
+		return 1;
 	}
 
-	obj = syscount_bpf__open();
+	obj = syscount_bpf__open_opts(&open_opts);
 	if (!obj) {
 		warn("failed to open BPF object\n");
 		err = 1;
@@ -416,6 +429,8 @@ int main(int argc, char **argv)
 		obj->rodata->count_by_process = true;
 	if (env.filter_errno)
 		obj->rodata->filter_errno = env.filter_errno;
+	if (env.cg)
+		obj->rodata->filter_cg = env.cg;
 
 	err = syscount_bpf__load(obj);
 	if (err) {
@@ -423,17 +438,31 @@ int main(int argc, char **argv)
 		goto cleanup_obj;
 	}
 
+	/* update cgroup path fd to map */
+	if (env.cg) {
+		idx = 0;
+		cg_map_fd = bpf_map__fd(obj->maps.cgroup_map);
+		cgfd = open(env.cgroupspath, O_RDONLY);
+		if (cgfd < 0) {
+			fprintf(stderr, "Failed opening Cgroup path: %s", env.cgroupspath);
+			goto cleanup_obj;
+		}
+		if (bpf_map_update_elem(cg_map_fd, &idx, &cgfd, BPF_ANY)) {
+			fprintf(stderr, "Failed adding target cgroup to map");
+			goto cleanup_obj;
+		}
+	}
+
 	obj->links.sys_exit = bpf_program__attach(obj->progs.sys_exit);
-	err = libbpf_get_error(obj->links.sys_exit);
-	if (err) {
-		warn("failed to attach sys_exit program: %s\n",
-		     strerror(-err));
+	if (!obj->links.sys_exit) {
+		err = -errno;
+		warn("failed to attach sys_exit program: %s\n", strerror(-err));
 		goto cleanup_obj;
 	}
 	if (env.latency) {
 		obj->links.sys_enter = bpf_program__attach(obj->progs.sys_enter);
-		err = libbpf_get_error(obj->links.sys_enter);
-		if (err) {
+		if (!obj->links.sys_enter) {
+			err = -errno;
 			warn("failed to attach sys_enter programs: %s\n",
 			     strerror(-err));
 			goto cleanup_obj;
@@ -474,6 +503,9 @@ cleanup_obj:
 	syscount_bpf__destroy(obj);
 free_names:
 	free_syscall_names();
+	cleanup_core_btf(&open_opts);
+	if (cgfd > 0)
+		close(cgfd);
 
 	return err != 0;
 }
