@@ -2,7 +2,7 @@
 # Copyright (c) PLUMgrid, Inc.
 # Licensed under the Apache License, Version 2.0 (the "License")
 
-from bcc import BPF
+from bcc import BPF, BPFAttachType, BPFProgType
 from bcc.libbcc import lib
 import ctypes as ct
 from unittest import main, skipUnless, TestCase
@@ -27,10 +27,10 @@ def redirect_stderr(to):
 
 class TestClang(TestCase):
     def test_complex(self):
-        b = BPF(src_file="test_clang_complex.c", debug=0)
-        fn = b.load_func("handle_packet", BPF.SCHED_CLS)
+        b = BPF(src_file=b"test_clang_complex.c", debug=0)
+        fn = b.load_func(b"handle_packet", BPF.SCHED_CLS)
     def test_printk(self):
-        text = """
+        text = b"""
 #include <bcc/proto.h>
 int handle_packet(void *ctx) {
   u8 *cursor = 0;
@@ -41,10 +41,10 @@ int handle_packet(void *ctx) {
 }
 """
         b = BPF(text=text, debug=0)
-        fn = b.load_func("handle_packet", BPF.SCHED_CLS)
+        fn = b.load_func(b"handle_packet", BPF.SCHED_CLS)
 
     def test_probe_read1(self):
-        text = """
+        text = b"""
 #include <linux/sched.h>
 #include <uapi/linux/ptrace.h>
 int count_sched(struct pt_regs *ctx, struct task_struct *prev) {
@@ -53,10 +53,20 @@ int count_sched(struct pt_regs *ctx, struct task_struct *prev) {
 }
 """
         b = BPF(text=text, debug=0)
-        fn = b.load_func("count_sched", BPF.KPROBE)
+        fn = b.load_func(b"count_sched", BPF.KPROBE)
+
+    def test_load_cgroup_sockopt_prog(self):
+        text = b"""
+int sockopt(struct bpf_sockopt* ctx){
+
+    return 0;
+}
+"""
+        b = BPF(text=text, debug=0)
+        fn =  b.load_func(b"sockopt", BPFProgType.CGROUP_SOCKOPT, device = None, attach_type = BPFAttachType.CGROUP_SETSOCKOPT)
 
     def test_probe_read2(self):
-        text = """
+        text = b"""
 #include <linux/sched.h>
 #include <uapi/linux/ptrace.h>
 int count_foo(struct pt_regs *ctx, unsigned long a, unsigned long b) {
@@ -64,10 +74,10 @@ int count_foo(struct pt_regs *ctx, unsigned long a, unsigned long b) {
 }
 """
         b = BPF(text=text, debug=0)
-        fn = b.load_func("count_foo", BPF.KPROBE)
+        fn = b.load_func(b"count_foo", BPF.KPROBE)
 
     def test_probe_read3(self):
-        text = """
+        text = b"""
 #include <net/tcp.h>
 #define _(P) ({typeof(P) val = 0; bpf_probe_read_kernel(&val, sizeof(val), &P); val;})
 int count_tcp(struct pt_regs *ctx, struct sk_buff *skb) {
@@ -75,10 +85,10 @@ int count_tcp(struct pt_regs *ctx, struct sk_buff *skb) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("count_tcp", BPF.KPROBE)
+        fn = b.load_func(b"count_tcp", BPF.KPROBE)
 
     def test_probe_read4(self):
-        text = """
+        text = b"""
 #include <net/tcp.h>
 #define _(P) ({typeof(P) val = 0; bpf_probe_read_kernel(&val, sizeof(val), &P); val;})
 int test(struct pt_regs *ctx, struct sk_buff *skb) {
@@ -86,10 +96,10 @@ int test(struct pt_regs *ctx, struct sk_buff *skb) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_whitelist1(self):
-        text = """
+        text = b"""
 #include <net/tcp.h>
 int count_tcp(struct pt_regs *ctx, struct sk_buff *skb) {
     // The below define is in net/tcp.h:
@@ -103,10 +113,10 @@ int count_tcp(struct pt_regs *ctx, struct sk_buff *skb) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("count_tcp", BPF.KPROBE)
+        fn = b.load_func(b"count_tcp", BPF.KPROBE)
 
     def test_probe_read_whitelist2(self):
-        text = """
+        text = b"""
 #include <net/tcp.h>
 int count_tcp(struct pt_regs *ctx, struct sk_buff *skb) {
     // The below define is in net/tcp.h:
@@ -120,10 +130,10 @@ int count_tcp(struct pt_regs *ctx, struct sk_buff *skb) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("count_tcp", BPF.KPROBE)
+        fn = b.load_func(b"count_tcp", BPF.KPROBE)
 
     def test_probe_read_keys(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/blkdev.h>
 BPF_HASH(start, struct request *);
@@ -146,15 +156,15 @@ int do_completion(struct pt_regs *ctx, struct request *req) {
 
     @skipUnless(lib.bpf_module_rw_engine_enabled(), "requires enabled rwengine")
     def test_sscanf(self):
-        text = """
+        text = b"""
 BPF_HASH(stats, int, struct { u64 a; u64 b; u64 c:36; u64 d:28; struct { u32 a; u32 b; } s; }, 10);
 int foo(void *ctx) {
     return 0;
 }
 """
         b = BPF(text=text, debug=0)
-        fn = b.load_func("foo", BPF.KPROBE)
-        t = b.get_table("stats")
+        fn = b.load_func(b"foo", BPF.KPROBE)
+        t = b.get_table(b"stats")
         s1 = t.key_sprintf(t.Key(2))
         self.assertEqual(s1, b"0x2")
         s2 = t.leaf_sprintf(t.Leaf(2, 3, 4, 1, (5, 6)))
@@ -168,11 +178,11 @@ int foo(void *ctx) {
 
     @skipUnless(lib.bpf_module_rw_engine_enabled(), "requires enabled rwengine")
     def test_sscanf_array(self):
-        text = """
+        text = b"""
 BPF_HASH(stats, int, struct { u32 a[3]; u32 b; }, 10);
 """
         b = BPF(text=text, debug=0)
-        t = b.get_table("stats")
+        t = b.get_table(b"stats")
         s1 = t.key_sprintf(t.Key(2))
         self.assertEqual(s1, b"0x2")
         s2 = t.leaf_sprintf(t.Leaf((ct.c_uint * 3)(1,2,3), 4))
@@ -185,7 +195,7 @@ BPF_HASH(stats, int, struct { u32 a[3]; u32 b; }, 10);
 
     @skipUnless(lib.bpf_module_rw_engine_enabled(), "requires enabled rwengine")
     def test_sscanf_string(self):
-        text = """
+        text = b"""
 struct Symbol {
     char name[128];
     char path[128];
@@ -198,7 +208,7 @@ struct Event {
 BPF_TABLE("array", int, struct Event, comms, 1);
 """
         b = BPF(text=text)
-        t = b.get_table("comms")
+        t = b.get_table(b"comms")
         s1 = t.leaf_sprintf(t[0])
         fill = b' { "" "" }' * 63
         self.assertEqual(s1, b'{ 0x0 0x0 [ { "" "" }%s ] }' % fill)
@@ -217,7 +227,7 @@ BPF_TABLE("array", int, struct Event, comms, 1);
         self.assertEqual(l.stack[0].path, path)
 
     def test_iosnoop(self):
-        text = """
+        text = b"""
 #include <linux/blkdev.h>
 #include <uapi/linux/ptrace.h>
 
@@ -235,10 +245,10 @@ int do_request(struct pt_regs *ctx, struct request *req) {
 }
 """
         b = BPF(text=text, debug=0)
-        fn = b.load_func("do_request", BPF.KPROBE)
+        fn = b.load_func(b"do_request", BPF.KPROBE)
 
     def test_blk_start_request(self):
-        text = """
+        text = b"""
 #include <linux/blkdev.h>
 #include <uapi/linux/ptrace.h>
 int do_request(struct pt_regs *ctx, int req) {
@@ -247,10 +257,10 @@ int do_request(struct pt_regs *ctx, int req) {
 }
 """
         b = BPF(text=text, debug=0)
-        fn = b.load_func("do_request", BPF.KPROBE)
+        fn = b.load_func(b"do_request", BPF.KPROBE)
 
     def test_bpf_hash(self):
-        text = """
+        text = b"""
 BPF_HASH(table1);
 BPF_HASH(table2, u32);
 BPF_HASH(table3, u32, int);
@@ -258,7 +268,7 @@ BPF_HASH(table3, u32, int);
         b = BPF(text=text, debug=0)
 
     def test_consecutive_probe_read(self):
-        text = """
+        text = b"""
 #include <linux/fs.h>
 #include <linux/mount.h>
 BPF_HASH(table1, struct super_block *);
@@ -277,10 +287,10 @@ int trace_entry(struct pt_regs *ctx, struct file *file) {
 }
 """
         b = BPF(text=text, debug=0)
-        fn = b.load_func("trace_entry", BPF.KPROBE)
+        fn = b.load_func(b"trace_entry", BPF.KPROBE)
 
     def test_nested_probe_read(self):
-        text = """
+        text = b"""
 #include <linux/fs.h>
 int trace_entry(struct pt_regs *ctx, struct file *file) {
     if (!file) return 0;
@@ -290,10 +300,10 @@ int trace_entry(struct pt_regs *ctx, struct file *file) {
 }
 """
         b = BPF(text=text, debug=0)
-        fn = b.load_func("trace_entry", BPF.KPROBE)
+        fn = b.load_func(b"trace_entry", BPF.KPROBE)
 
     def test_nested_probe_read_deref(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 struct sock {
     u32 *sk_daddr;
@@ -303,10 +313,10 @@ int test(struct pt_regs *ctx, struct sock *skp) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_char_array_probe(self):
-        BPF(text="""#include <linux/blkdev.h>
+        BPF(text=b"""#include <linux/blkdev.h>
 int kprobe__blk_update_request(struct pt_regs *ctx, struct request *req) {
     bpf_trace_printk("%s\\n", req->rq_disk->disk_name);
     return 0;
@@ -317,13 +327,13 @@ int kprobe__blk_update_request(struct pt_regs *ctx, struct request *req) {
         # Skip if the kernel is not compiled with CONFIG_BPF_LSM
         if not BPF.support_lsm():
             return
-        b = BPF(text="""
+        b = BPF(text=b"""
 LSM_PROBE(bpf, int cmd, union bpf_attr *uattr, unsigned int size) {
     return 0;
 }""")
 
     def test_probe_read_helper(self):
-        b = BPF(text="""
+        b = BPF(text=b"""
 #include <linux/fs.h>
 static void print_file_name(struct file *file) {
     if (!file) return;
@@ -342,11 +352,11 @@ int trace_entry2(struct pt_regs *ctx, int unused, struct file *file) {
     return 0;
 }
 """)
-        fn = b.load_func("trace_entry1", BPF.KPROBE)
-        fn = b.load_func("trace_entry2", BPF.KPROBE)
+        fn = b.load_func(b"trace_entry1", BPF.KPROBE)
+        fn = b.load_func(b"trace_entry2", BPF.KPROBE)
 
     def test_probe_unnamed_union_deref(self):
-        text = """
+        text = b"""
 #include <linux/mm_types.h>
 int trace(struct pt_regs *ctx, struct page *page) {
     void *p = page->mapping;
@@ -360,7 +370,7 @@ int trace(struct pt_regs *ctx, struct page *page) {
             pass
 
     def test_probe_struct_assign(self):
-        b = BPF(text = """
+        b = BPF(text = b"""
 #include <uapi/linux/ptrace.h>
 struct args_t {
     const char *filename;
@@ -377,11 +387,11 @@ int do_sys_open(struct pt_regs *ctx, const char *filename,
     return 0;
 };
 """)
-        b.attach_kprobe(event=b.get_syscall_fnname("open"),
-                        fn_name="do_sys_open")
+        b.attach_kprobe(event=b.get_syscall_fnname(b"open"),
+                        fn_name=b"do_sys_open")
 
     def test_task_switch(self):
-        b = BPF(text="""
+        b = BPF(text=b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/sched.h>
 struct key_t {
@@ -404,7 +414,7 @@ int kprobe__finish_task_switch(struct pt_regs *ctx, struct task_struct *prev) {
 """)
 
     def test_probe_simple_assign(self):
-        b = BPF(text="""
+        b = BPF(text=b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/gfp.h>
 struct leaf { size_t size; };
@@ -418,7 +428,7 @@ int kprobe____kmalloc(struct pt_regs *ctx, size_t size) {
 }""")
 
     def test_probe_simple_member_assign(self):
-        b = BPF(text="""
+        b = BPF(text=b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/netdevice.h>
 struct leaf { void *ptr; };
@@ -428,10 +438,10 @@ int test(struct pt_regs *ctx, struct sk_buff *skb) {
     lp->ptr = skb;
     return 0;
 }""")
-        b.load_func("test", BPF.KPROBE)
+        b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_member_expr_deref(self):
-        b = BPF(text="""
+        b = BPF(text=b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/netdevice.h>
 struct leaf { struct sk_buff *ptr; };
@@ -441,10 +451,10 @@ int test(struct pt_regs *ctx, struct sk_buff *skb) {
     lp->ptr = skb;
     return lp->ptr->priority;
 }""")
-        b.load_func("test", BPF.KPROBE)
+        b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_member_expr(self):
-        b = BPF(text="""
+        b = BPF(text=b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/netdevice.h>
 struct leaf { struct sk_buff *ptr; };
@@ -454,10 +464,10 @@ int test(struct pt_regs *ctx, struct sk_buff *skb) {
     lp->ptr = skb;
     return l.ptr->priority;
 }""")
-        b.load_func("test", BPF.KPROBE)
+        b.load_func(b"test", BPF.KPROBE)
 
     def test_unop_probe_read(self):
-        text = """
+        text = b"""
 #include <linux/blkdev.h>
 int trace_entry(struct pt_regs *ctx, struct request *req) {
     if (!(req->bio->bi_flags & 1))
@@ -468,10 +478,10 @@ int trace_entry(struct pt_regs *ctx, struct request *req) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("trace_entry", BPF.KPROBE)
+        fn = b.load_func(b"trace_entry", BPF.KPROBE)
 
     def test_probe_read_nested_deref(self):
-        text = """
+        text = b"""
 #include <net/inet_sock.h>
 int test(struct pt_regs *ctx, struct sock *sk) {
     struct sock *ptr1;
@@ -481,10 +491,10 @@ int test(struct pt_regs *ctx, struct sock *sk) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_nested_deref2(self):
-        text = """
+        text = b"""
 #include <net/inet_sock.h>
 int test(struct pt_regs *ctx, struct sock *sk) {
     struct sock *ptr1;
@@ -496,10 +506,10 @@ int test(struct pt_regs *ctx, struct sock *sk) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_nested_deref3(self):
-        text = """
+        text = b"""
 #include <net/inet_sock.h>
 int test(struct pt_regs *ctx, struct sock *sk) {
     struct sock **ptr1, **ptr2 = &sk;
@@ -508,10 +518,10 @@ int test(struct pt_regs *ctx, struct sock *sk) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_nested_deref_func1(self):
-        text = """
+        text = b"""
 #include <net/inet_sock.h>
 static struct sock **subtest(struct sock **sk) {
     return sk;
@@ -523,10 +533,10 @@ int test(struct pt_regs *ctx, struct sock *sk) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_nested_deref_func2(self):
-        text = """
+        text = b"""
 #include <net/inet_sock.h>
 static int subtest(struct sock ***skp) {
     return ((struct sock *)(**skp))->sk_daddr;
@@ -541,10 +551,10 @@ int test(struct pt_regs *ctx, struct sock *sk) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_nested_member1(self):
-        text = """
+        text = b"""
 #include <net/inet_sock.h>
 int test(struct pt_regs *ctx, struct sock *skp) {
     u32 *daddr = &skp->sk_daddr;
@@ -552,10 +562,10 @@ int test(struct pt_regs *ctx, struct sock *skp) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_nested_member2(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 struct sock {
     u32 **sk_daddr;
@@ -566,10 +576,10 @@ int test(struct pt_regs *ctx, struct sock *skp) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_nested_member3(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 struct sock {
     u32 *sk_daddr;
@@ -579,10 +589,10 @@ int test(struct pt_regs *ctx, struct sock *skp) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_paren_probe_read(self):
-        text = """
+        text = b"""
 #include <net/inet_sock.h>
 int trace_entry(struct pt_regs *ctx, struct sock *sk) {
     u16 sport = ((struct inet_sock *)sk)->inet_sport;
@@ -590,10 +600,10 @@ int trace_entry(struct pt_regs *ctx, struct sock *sk) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("trace_entry", BPF.KPROBE)
+        fn = b.load_func(b"trace_entry", BPF.KPROBE)
 
     def test_complex_leaf_types(self):
-        text = """
+        text = b"""
 struct list;
 struct list {
   struct list *selfp;
@@ -613,10 +623,10 @@ BPF_ARRAY(t2, struct list *, 1);
 BPF_ARRAY(t3, union emptyu, 1);
 """
         b = BPF(text=text)
-        self.assertEqual(ct.sizeof(b["t3"].Leaf), 8)
+        self.assertEqual(ct.sizeof(b[b"t3"].Leaf), 8)
 
     def test_cflags(self):
-        text = """
+        text = b"""
 #ifndef MYFLAG
 #error "MYFLAG not set as expected"
 #endif
@@ -624,24 +634,24 @@ BPF_ARRAY(t3, union emptyu, 1);
         b = BPF(text=text, cflags=["-DMYFLAG"])
 
     def test_exported_maps(self):
-        b1 = BPF(text="""BPF_TABLE_PUBLIC("hash", int, int, table1, 10);""")
-        b2 = BPF(text="""BPF_TABLE("extern", int, int, table1, 10);""")
-        t = b2["table1"]
+        b1 = BPF(text=b"""BPF_TABLE_PUBLIC("hash", int, int, table1, 10);""")
+        b2 = BPF(text=b"""BPF_TABLE("extern", int, int, table1, 10);""")
+        t = b2[b"table1"]
 
     def test_syntax_error(self):
         with self.assertRaises(Exception):
-            b = BPF(text="""int failure(void *ctx) { if (); return 0; }""")
+            b = BPF(text=b"""int failure(void *ctx) { if (); return 0; }""")
 
     def test_nested_union(self):
-        text = """
+        text = b"""
 BPF_HASH(t1, struct bpf_tunnel_key, int, 1);
 """
         b = BPF(text=text)
-        t1 = b["t1"]
+        t1 = b[b"t1"]
         print(t1.Key().remote_ipv4)
 
     def test_too_many_args(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 int many(struct pt_regs *ctx, int a, int b, int c, int d, int e, int f, int g) {
     return 0;
@@ -651,7 +661,7 @@ int many(struct pt_regs *ctx, int a, int b, int c, int d, int e, int f, int g) {
             b = BPF(text=text)
 
     def test_call_macro_arg(self):
-        text = """
+        text = b"""
 BPF_PROG_ARRAY(jmp, 32);
 
 #define JMP_IDX_PIPE (1U << 1)
@@ -667,11 +677,11 @@ int process(struct xdp_md *ctx) {
 }
         """
         b = BPF(text=text)
-        t = b["jmp"]
+        t = b[b"jmp"]
         self.assertEqual(len(t), 32);
 
     def test_update_macro_arg(self):
-        text = """
+        text = b"""
 BPF_ARRAY(act, u32, 32);
 
 #define JMP_IDX_PIPE (1U << 1)
@@ -687,11 +697,11 @@ int process(struct xdp_md *ctx) {
 }
         """
         b = BPF(text=text)
-        t = b["act"]
+        t = b[b"act"]
         self.assertEqual(len(t), 32);
 
     def test_ext_ptr_maps1(self):
-        bpf_text = """
+        bpf_text = b"""
 #include <uapi/linux/ptrace.h>
 #include <net/sock.h>
 #include <bcc/proto.h>
@@ -717,11 +727,11 @@ int trace_exit(struct pt_regs *ctx) {
 }
         """
         b = BPF(text=bpf_text)
-        b.load_func("trace_entry", BPF.KPROBE)
-        b.load_func("trace_exit", BPF.KPROBE)
+        b.load_func(b"trace_entry", BPF.KPROBE)
+        b.load_func(b"trace_exit", BPF.KPROBE)
 
     def test_ext_ptr_maps2(self):
-        bpf_text = """
+        bpf_text = b"""
 #include <uapi/linux/ptrace.h>
 #include <net/sock.h>
 #include <bcc/proto.h>
@@ -746,11 +756,11 @@ int trace_exit(struct pt_regs *ctx) {
 }
         """
         b = BPF(text=bpf_text)
-        b.load_func("trace_entry", BPF.KPROBE)
-        b.load_func("trace_exit", BPF.KPROBE)
+        b.load_func(b"trace_entry", BPF.KPROBE)
+        b.load_func(b"trace_exit", BPF.KPROBE)
 
     def test_ext_ptr_maps_reverse(self):
-        bpf_text = """
+        bpf_text = b"""
 #include <uapi/linux/ptrace.h>
 #include <net/sock.h>
 #include <bcc/proto.h>
@@ -775,11 +785,11 @@ int trace_entry(struct pt_regs *ctx, struct sock *sk) {
 };
         """
         b = BPF(text=bpf_text)
-        b.load_func("trace_entry", BPF.KPROBE)
-        b.load_func("trace_exit", BPF.KPROBE)
+        b.load_func(b"trace_entry", BPF.KPROBE)
+        b.load_func(b"trace_exit", BPF.KPROBE)
 
     def test_ext_ptr_maps_indirect(self):
-        bpf_text = """
+        bpf_text = b"""
 #include <uapi/linux/ptrace.h>
 #include <net/sock.h>
 #include <bcc/proto.h>
@@ -805,11 +815,11 @@ int trace_exit(struct pt_regs *ctx) {
 }
         """
         b = BPF(text=bpf_text)
-        b.load_func("trace_entry", BPF.KPROBE)
-        b.load_func("trace_exit", BPF.KPROBE)
+        b.load_func(b"trace_entry", BPF.KPROBE)
+        b.load_func(b"trace_exit", BPF.KPROBE)
 
     def test_bpf_dins_pkt_rewrite(self):
-        text = """
+        text = b"""
 #include <bcc/proto.h>
 int dns_test(struct __sk_buff *skb) {
     u8 *cursor = 0;
@@ -826,7 +836,7 @@ int dns_test(struct __sk_buff *skb) {
 
     @skipUnless(kernel_version_ge(4,8), "requires kernel >= 4.8")
     def test_ext_ptr_from_helper(self):
-        text = """
+        text = b"""
 #include <linux/sched.h>
 int test(struct pt_regs *ctx) {
     struct task_struct *task = (struct task_struct *)bpf_get_current_task();
@@ -834,10 +844,10 @@ int test(struct pt_regs *ctx) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_unary_operator(self):
-        text = """
+        text = b"""
 #include <linux/fs.h>
 #include <uapi/linux/ptrace.h>
 int trace_read_entry(struct pt_regs *ctx, struct file *file) {
@@ -846,13 +856,13 @@ int trace_read_entry(struct pt_regs *ctx, struct file *file) {
         """
         b = BPF(text=text)
         try:
-            b.attach_kprobe(event="__vfs_read", fn_name="trace_read_entry")
+            b.attach_kprobe(event=b"__vfs_read", fn_name=b"trace_read_entry")
         except Exception:
             print('Current kernel does not have __vfs_read, try vfs_read instead')
-            b.attach_kprobe(event="vfs_read", fn_name="trace_read_entry")
+            b.attach_kprobe(event=b"vfs_read", fn_name=b"trace_read_entry")
 
     def test_printk_f(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 int trace_entry(struct pt_regs *ctx) {
   bpf_trace_printk("%0.2f\\n", 1);
@@ -869,7 +879,7 @@ int trace_entry(struct pt_regs *ctx) {
         r.close()
 
     def test_printk_lf(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 int trace_entry(struct pt_regs *ctx) {
   bpf_trace_printk("%lf\\n", 1);
@@ -886,7 +896,7 @@ int trace_entry(struct pt_regs *ctx) {
         r.close()
 
     def test_printk_2s(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 int trace_entry(struct pt_regs *ctx) {
   char s1[] = "hello", s2[] = "world";
@@ -904,7 +914,7 @@ int trace_entry(struct pt_regs *ctx) {
         r.close()
 
     def test_map_insert(self):
-        text = """
+        text = b"""
 BPF_HASH(dummy);
 void do_trace(struct pt_regs *ctx) {
     u64 key = 0, val = 2;
@@ -915,34 +925,34 @@ void do_trace(struct pt_regs *ctx) {
 """
         b = BPF(text=text)
         c_val = ct.c_ulong(1)
-        b["dummy"][ct.c_ulong(0)] = c_val
-        b["dummy"][ct.c_ulong(1)] = c_val
-        b.attach_kprobe(event=b.get_syscall_fnname("sync"), fn_name="do_trace")
+        b[b"dummy"][ct.c_ulong(0)] = c_val
+        b[b"dummy"][ct.c_ulong(1)] = c_val
+        b.attach_kprobe(event=b.get_syscall_fnname(b"sync"), fn_name=b"do_trace")
         libc = ct.CDLL("libc.so.6")
         libc.sync()
-        self.assertEqual(1, b["dummy"][ct.c_ulong(0)].value)
-        self.assertEqual(2, b["dummy"][ct.c_ulong(1)].value)
+        self.assertEqual(1, b[b"dummy"][ct.c_ulong(0)].value)
+        self.assertEqual(2, b[b"dummy"][ct.c_ulong(1)].value)
 
     def test_prog_array_delete(self):
-        text = """
+        text = b"""
 BPF_PROG_ARRAY(dummy, 256);
 """
         b1 = BPF(text=text)
-        text = """
+        text = b"""
 int do_next(struct pt_regs *ctx) {
     return 0;
 }
 """
         b2 = BPF(text=text)
-        fn = b2.load_func("do_next", BPF.KPROBE)
+        fn = b2.load_func(b"do_next", BPF.KPROBE)
         c_key = ct.c_int(0)
-        b1["dummy"][c_key] = ct.c_int(fn.fd)
-        b1["dummy"].__delitem__(c_key);
+        b1[b"dummy"][c_key] = ct.c_int(fn.fd)
+        b1[b"dummy"].__delitem__(c_key);
         with self.assertRaises(KeyError):
-            b1["dummy"][c_key]
+            b1[b"dummy"][c_key]
 
     def test_invalid_noninline_call(self):
-        text = """
+        text = b"""
 int bar(void) {
     return 0;
 }
@@ -954,7 +964,7 @@ int foo(struct pt_regs *ctx) {
             b = BPF(text=text)
 
     def test_incomplete_type(self):
-        text = """
+        text = b"""
 BPF_HASH(drops, struct key_t);
 struct key_t {
     u64 location;
@@ -964,7 +974,7 @@ struct key_t {
             b = BPF(text=text)
 
     def test_enumerations(self):
-        text = """
+        text = b"""
 enum b {
     CHOICE_A,
 };
@@ -974,14 +984,14 @@ struct a {
 BPF_HASH(drops, struct a);
         """
         b = BPF(text=text)
-        t = b['drops']
+        t = b[b'drops']
 
     def test_int128_types(self):
-        text = """
+        text = b"""
 BPF_HASH(table1, unsigned __int128, __int128);
 """
         b = BPF(text=text)
-        table = b['table1']
+        table = b[b'table1']
         self.assertEqual(ct.sizeof(table.Key), 16)
         self.assertEqual(ct.sizeof(table.Leaf), 16)
         table[
@@ -996,7 +1006,7 @@ BPF_HASH(table1, unsigned __int128, __int128);
                              "2001:db8::")
 
     def test_padding_types(self):
-        text = """
+        text = b"""
 struct key_t {
   u32 f1_1;               /* offset 0 */
   struct {
@@ -1014,13 +1024,13 @@ struct value_t {
 BPF_HASH(table1, struct key_t, struct value_t);
 """
         b = BPF(text=text)
-        table = b['table1']
+        table = b[b'table1']
         self.assertEqual(ct.sizeof(table.Key), 96)
         self.assertEqual(ct.sizeof(table.Leaf), 16)
 
     @skipUnless(kernel_version_ge(4,7), "requires kernel >= 4.7")
     def test_probe_read_tracepoint_context(self):
-        text = """
+        text = b"""
 #include <linux/netdevice.h>
 TRACEPOINT_PROBE(skb, kfree_skb) {
     struct sk_buff *skb = (struct sk_buff *)args->skbaddr;
@@ -1030,7 +1040,7 @@ TRACEPOINT_PROBE(skb, kfree_skb) {
         b = BPF(text=text)
 
     def test_probe_read_kprobe_ctx(self):
-        text = """
+        text = b"""
 #include <linux/sched.h>
 #include <net/inet_sock.h>
 int test(struct pt_regs *ctx) {
@@ -1040,10 +1050,10 @@ int test(struct pt_regs *ctx) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_ctx_array(self):
-        text = """
+        text = b"""
 #include <linux/sched.h>
 #include <net/inet_sock.h>
 int test(struct pt_regs *ctx) {
@@ -1052,11 +1062,11 @@ int test(struct pt_regs *ctx) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     @skipUnless(kernel_version_ge(4,7), "requires kernel >= 4.7")
     def test_probe_read_tc_ctx(self):
-        text = """
+        text = b"""
 #include <uapi/linux/pkt_cls.h>
 #include <linux/if_ether.h>
 int test(struct __sk_buff *ctx) {
@@ -1071,10 +1081,10 @@ int test(struct __sk_buff *ctx) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.SCHED_CLS)
+        fn = b.load_func(b"test", BPF.SCHED_CLS)
 
     def test_probe_read_return(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/tcp.h>
 static inline unsigned char *my_skb_transport_header(struct sk_buff *skb) {
@@ -1086,10 +1096,10 @@ int test(struct pt_regs *ctx, struct sock *sk, struct sk_buff *skb) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_multiple_return(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/tcp.h>
 static inline u64 error_function() {
@@ -1106,10 +1116,10 @@ int test(struct pt_regs *ctx, struct sock *sk, struct sk_buff *skb) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_return_expr(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/tcp.h>
 static inline unsigned char *my_skb_transport_header(struct sk_buff *skb) {
@@ -1121,10 +1131,10 @@ int test(struct pt_regs *ctx, struct sock *sk, struct sk_buff *skb) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_return_call(self):
-        text = """
+        text = b"""
 #include <uapi/linux/ptrace.h>
 #include <linux/tcp.h>
 static inline struct tcphdr *my_skb_transport_header(struct sk_buff *skb) {
@@ -1135,10 +1145,10 @@ int test(struct pt_regs *ctx, struct sock *sk, struct sk_buff *skb) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_no_probe_read_addrof(self):
-        text = """
+        text = b"""
 #include <linux/sched.h>
 #include <net/inet_sock.h>
 static inline int test_help(__be16 *addr) {
@@ -1153,10 +1163,10 @@ int test(struct pt_regs *ctx) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_array_accesses1(self):
-        text = """
+        text = b"""
 #include <linux/ptrace.h>
 #include <linux/dcache.h>
 int test(struct pt_regs *ctx, const struct qstr *name) {
@@ -1164,10 +1174,10 @@ int test(struct pt_regs *ctx, const struct qstr *name) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_array_accesses2(self):
-        text = """
+        text = b"""
 #include <linux/ptrace.h>
 #include <linux/dcache.h>
 int test(struct pt_regs *ctx, const struct qstr *name) {
@@ -1175,10 +1185,10 @@ int test(struct pt_regs *ctx, const struct qstr *name) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_array_accesses3(self):
-        text = """
+        text = b"""
 #include <linux/ptrace.h>
 #include <linux/dcache.h>
 int test(struct pt_regs *ctx, const struct qstr *name) {
@@ -1186,30 +1196,30 @@ int test(struct pt_regs *ctx, const struct qstr *name) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_array_accesses4(self):
-        text = """
+        text = b"""
 #include <linux/ptrace.h>
 int test(struct pt_regs *ctx, char *name) {
     return name[1];
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_array_accesses5(self):
-        text = """
+        text = b"""
 #include <linux/ptrace.h>
 int test(struct pt_regs *ctx, char **name) {
     return (*name)[1];
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_array_accesses6(self):
-        text = """
+        text = b"""
 #include <linux/ptrace.h>
 struct test_t {
     int tab[5];
@@ -1219,27 +1229,27 @@ int test(struct pt_regs *ctx, struct test_t *t) {
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_array_accesses7(self):
-        text = """
+        text = b"""
 #include <net/inet_sock.h>
 int test(struct pt_regs *ctx, struct sock *sk) {
     return sk->__sk_common.skc_v6_rcv_saddr.in6_u.u6_addr32[0];
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_probe_read_array_accesses8(self):
-        text = """
+        text = b"""
 #include <linux/mm_types.h>
 int test(struct pt_regs *ctx, struct mm_struct *mm) {
     return mm->rss_stat.count[MM_ANONPAGES].counter;
 }
 """
         b = BPF(text=text)
-        fn = b.load_func("test", BPF.KPROBE)
+        fn = b.load_func(b"test", BPF.KPROBE)
 
     def test_arbitrary_increment_simple(self):
         b = BPF(text=b"""
@@ -1275,14 +1285,14 @@ TRACEPOINT_PROBE(kmem, kmalloc) {
     return 0;
 }
 """)
-        if len(b["testing"].items()):
-            st = b["testing"][ct.c_uint(0)]
+        if len(b[b"testing"].items()):
+            st = b[b"testing"][ct.c_uint(0)]
             self.assertEqual(st.a, 10)
             self.assertEqual(st.b, 20)
 
     @skipUnless(kernel_version_ge(4,14), "requires kernel >= 4.14")
     def test_jump_table(self):
-        text = """
+        text = b"""
 #include <linux/blk_types.h>
 #include <linux/blkdev.h>
 #include <linux/time64.h>
