@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # Copyright (c) PLUMgrid, Inc.
 # Licensed under the Apache License, Version 2.0 (the "License")
 
@@ -8,7 +8,7 @@ from time import sleep
 import sys
 from unittest import main, TestCase
 
-text = """
+text = b"""
 #include <linux/ptrace.h>
 struct Ptr { u64 ptr; };
 struct Counters { char unused; __int128 stat1; };
@@ -30,8 +30,9 @@ int count_sched(struct pt_regs *ctx) {
 class TestTracingEvent(TestCase):
     def setUp(self):
         b = BPF(text=text, debug=0)
-        self.stats = b.get_table("stats")
-        b.attach_kprobe(event="finish_task_switch", fn_name="count_sched")
+        self.stats = b.get_table(b"stats")
+        b.attach_kprobe(event_re=b"^finish_task_switch$|^finish_task_switch\.isra\.\d$",
+                        fn_name=b"count_sched")
 
     def test_sched1(self):
         for i in range(0, 100):
