@@ -1,21 +1,23 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 # @lint-avoid-python-3-compatibility-imports
 #
 # biolatency    Summarize block device I/O latency as a histogram.
 #       For Linux, uses BCC, eBPF.
 #
-# USAGE: biolatency [-h] [-T] [-Q] [-m] [-D] [-e] [interval] [count]
+# USAGE: biolatency [-h] [-T] [-Q] [-m] [-D] [-F] [-e] [-j] [-d DISK] [interval] [count]
 #
 # Copyright (c) 2015 Brendan Gregg.
 # Licensed under the Apache License, Version 2.0 (the "License")
 #
 # 20-Sep-2015   Brendan Gregg   Created this.
+# 31-Mar-2022   Rocky Xing      Added disk filter support.
 
 from __future__ import print_function
 from bcc import BPF
 from time import sleep, strftime
 import argparse
 import ctypes as ct
+import os
 
 # arguments
 examples = """examples:
@@ -27,6 +29,7 @@ examples = """examples:
     ./biolatency -F                 # show I/O flags separately
     ./biolatency -j                 # print a dictionary
     ./biolatency -e                 # show extension summary(total, average)
+    ./biolatency -d sdc             # Trace sdc only
 """
 parser = argparse.ArgumentParser(
     description="Summarize block device I/O latency as a histogram",
@@ -52,6 +55,8 @@ parser.add_argument("--ebpf", action="store_true",
     help=argparse.SUPPRESS)
 parser.add_argument("-j", "--json", action="store_true",
     help="json output")
+parser.add_argument("-d", "--disk", type=str,
+    help="Trace this disk only")
 
 args = parser.parse_args()
 countdown = int(args.count)
@@ -64,7 +69,7 @@ if args.flags and args.disks:
 # define BPF program
 bpf_text = """
 #include <uapi/linux/ptrace.h>
-#include <linux/blkdev.h>
+#include <linux/blk-mq.h>
 
 typedef struct disk_key {
     char disk[DISK_NAME_LEN];
@@ -87,6 +92,8 @@ STORAGE
 // time block I/O
 int trace_req_start(struct pt_regs *ctx, struct request *req)
 {
+    DISK_FILTER
+
     u64 ts = bpf_ktime_get_ns();
     start.update(&req, &ts);
     return 0;
@@ -104,8 +111,6 @@ int trace_req_done(struct pt_regs *ctx, struct request *req)
     }
     delta = bpf_ktime_get_ns() - *tsp;
 
-    EXTENSION
-
     FACTOR
 
     // store as histogram
@@ -128,12 +133,16 @@ storage_str = ""
 store_str = ""
 if args.disks:
     storage_str += "BPF_HISTOGRAM(dist, disk_key_t);"
-    store_str += """
+    disks_str = """
     disk_key_t key = {.slot = bpf_log2l(delta)};
-    void *__tmp = (void *)req->rq_disk->disk_name;
+    void *__tmp = (void *)req->__RQ_DISK__->disk_name;
     bpf_probe_read(&key.disk, sizeof(key.disk), __tmp);
     dist.atomic_increment(key);
     """
+    if BPF.kernel_struct_has_field(b'request', b'rq_disk') == 1:
+        store_str += disks_str.replace('__RQ_DISK__', 'rq_disk')
+    else:
+        store_str += disks_str.replace('__RQ_DISK__', 'q->disk')
 elif args.flags:
     storage_str += "BPF_HISTOGRAM(dist, flag_key_t);"
     store_str += """
@@ -145,18 +154,44 @@ else:
     storage_str += "BPF_HISTOGRAM(dist);"
     store_str += "dist.atomic_increment(bpf_log2l(delta));"
 
+if args.disk is not None:
+    disk_path = os.path.join('/dev', args.disk)
+    if not os.path.exists(disk_path):
+        print("no such disk '%s'" % args.disk)
+        exit(1)
+
+    stat_info = os.stat(disk_path)
+    major = os.major(stat_info.st_rdev)
+    minor = os.minor(stat_info.st_rdev)
+
+    disk_field_str = ""
+    if BPF.kernel_struct_has_field(b'request', b'rq_disk') == 1:
+        disk_field_str = 'req->rq_disk'
+    else:
+        disk_field_str = 'req->q->disk'
+
+    disk_filter_str = """
+    struct gendisk *disk = %s;
+    if (!(disk->major == %d && disk->first_minor == %d)) {
+        return 0;
+    }
+    """ % (disk_field_str, major, minor)
+
+    bpf_text = bpf_text.replace('DISK_FILTER', disk_filter_str)
+else:
+    bpf_text = bpf_text.replace('DISK_FILTER', '')
+
 if args.extension:
     storage_str += "BPF_ARRAY(extension, ext_val_t, 1);"
-    bpf_text = bpf_text.replace('EXTENSION', """
+    store_str += """
     u32 index = 0;
     ext_val_t *ext_val = extension.lookup(&index);
     if (ext_val) {
         lock_xadd(&ext_val->total, delta);
         lock_xadd(&ext_val->count, 1);
     }
-    """)
-else:
-    bpf_text = bpf_text.replace('EXTENSION', '')
+    """
+
 bpf_text = bpf_text.replace("STORAGE", storage_str)
 bpf_text = bpf_text.replace("STORE", store_str)
 
@@ -168,17 +203,28 @@ if debug or args.ebpf:
 # load BPF program
 b = BPF(text=bpf_text)
 if args.queued:
-    b.attach_kprobe(event="blk_account_io_start", fn_name="trace_req_start")
+    if BPF.get_kprobe_functions(b'__blk_account_io_start'):
+        b.attach_kprobe(event="__blk_account_io_start", fn_name="trace_req_start")
+    else:
+        b.attach_kprobe(event="blk_account_io_start", fn_name="trace_req_start")
 else:
     if BPF.get_kprobe_functions(b'blk_start_request'):
         b.attach_kprobe(event="blk_start_request", fn_name="trace_req_start")
     b.attach_kprobe(event="blk_mq_start_request", fn_name="trace_req_start")
-b.attach_kprobe(event="blk_account_io_done",
-    fn_name="trace_req_done")
+if BPF.get_kprobe_functions(b'__blk_account_io_done'):
+    b.attach_kprobe(event="__blk_account_io_done", fn_name="trace_req_done")
+else:
+    b.attach_kprobe(event="blk_account_io_done", fn_name="trace_req_done")
 
 if not args.json:
     print("Tracing block device I/O... Hit Ctrl-C to end.")
 
+def disk_print(s):
+    disk = s.decode('utf-8', 'replace')
+    if not disk:
+        disk = "<unknown>"
+    return disk
+
 # see blk_fill_rwbs():
 req_opf = {
     0: "Read",
@@ -247,9 +293,8 @@ while (1):
 
         if args.flags:
             dist.print_json_hist(label, "flags", flags_print)
-
         else:
-            dist.print_json_hist(label)
+            dist.print_json_hist(label, "disk", disk_print)
 
     else:
         if args.timestamp:
@@ -258,18 +303,13 @@ while (1):
         if args.flags:
             dist.print_log2_hist(label, "flags", flags_print)
         else:
-            dist.print_log2_hist(label, "disk")
+            dist.print_log2_hist(label, "disk", disk_print)
         if args.extension:
             total = extension[0].total
-            counts = extension[0].count
-            if counts > 0:
-                if label == 'msecs':
-                    total /= 1000000
-                elif label == 'usecs':
-                    total /= 1000
-                avg = total / counts
+            count = extension[0].count
+            if count > 0:
                 print("\navg = %ld %s, total: %ld %s, count: %ld\n" %
-                      (total / counts, label, total, label, counts))
+                      (total / count, label, total, label, count))
             extension.clear()
 
     dist.clear()
@@ -277,4 +317,3 @@ while (1):
     countdown -= 1
     if exiting or countdown == 0:
         exit()
-
