@@ -1,10 +1,10 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 # @lint-avoid-python-3-compatibility-imports
 #
 # biotop  block device (disk) I/O by process.
 #         For Linux, uses BCC, eBPF.
 #
-# USAGE: biotop.py [-h] [-C] [-r MAXROWS] [interval] [count]
+# USAGE: biotop.py [-h] [-C] [-r MAXROWS] [-p PID] [interval] [count]
 #
 # This uses in-kernel eBPF maps to cache process details (PID and comm) by I/O
 # request, as well as a starting timestamp for calculating I/O latency.
@@ -13,6 +13,7 @@
 # Licensed under the Apache License, Version 2.0 (the "License")
 #
 # 06-Feb-2016   Brendan Gregg   Created this.
+# 17-Mar-2022   Rocky Xing      Added PID filter support.
 
 from __future__ import print_function
 from bcc import BPF
@@ -24,6 +25,7 @@ from subprocess import call
 examples = """examples:
     ./biotop            # block device I/O top, 1 second refresh
     ./biotop -C         # don't clear the screen
+    ./biotop -p 181     # only trace PID 181
     ./biotop 5          # 5 second summaries
     ./biotop 5 10       # 5 second summaries, 10 times only
 """
@@ -35,6 +37,8 @@ parser.add_argument("-C", "--noclear", action="store_true",
     help="don't clear the screen")
 parser.add_argument("-r", "--maxrows", default=20,
     help="maximum rows to print, default 20")
+parser.add_argument("-p", "--pid", type=int, metavar="PID",
+    help="trace this PID only")
 parser.add_argument("interval", nargs="?", default=1,
     help="output interval, in seconds")
 parser.add_argument("count", nargs="?", default=99999999,
@@ -54,7 +58,7 @@ diskstats = "/proc/diskstats"
 # load BPF program
 bpf_text = """
 #include <uapi/linux/ptrace.h>
-#include <linux/blkdev.h>
+#include <linux/blk-mq.h>
 
 // for saving the timestamp and __data_len of each request
 struct start_req_t {
@@ -92,9 +96,14 @@ BPF_HASH(counts, struct info_t, struct val_t);
 int trace_pid_start(struct pt_regs *ctx, struct request *req)
 {
     struct who_t who = {};
+    u32 pid;
 
     if (bpf_get_current_comm(&who.name, sizeof(who.name)) == 0) {
-        who.pid = bpf_get_current_pid_tgid() >> 32;
+        pid = bpf_get_current_pid_tgid() >> 32;
+        if (FILTER_PID)
+            return 0;
+
+        who.pid = pid;
         whobyreq.update(&req, &who);
     }
 
@@ -124,13 +133,25 @@ int trace_req_completion(struct pt_regs *ctx, struct request *req)
     }
 
     struct who_t *whop;
+    u32 pid;
+
+    whop = whobyreq.lookup(&req);
+    pid = whop != 0 ? whop->pid : 0;
+    if (FILTER_PID) {
+        start.delete(&req);
+        if (whop != 0) {
+            whobyreq.delete(&req);
+        }
+        return 0;
+    }
+
     struct val_t *valp, zero = {};
     u64 delta_us = (bpf_ktime_get_ns() - startp->ts) / 1000;
 
     // setup info_t key
     struct info_t info = {};
-    info.major = req->rq_disk->major;
-    info.minor = req->rq_disk->first_minor;
+    info.major = req->__RQ_DISK__->major;
+    info.minor = req->__RQ_DISK__->first_minor;
 /*
  * The following deals with a kernel version change (in mainline 4.7, although
  * it may be backported to earlier kernels) with how block request write flags
@@ -146,7 +167,6 @@ int trace_req_completion(struct pt_regs *ctx, struct request *req)
     info.rwflag = !!((req->cmd_flags & REQ_OP_MASK) == REQ_OP_WRITE);
 #endif
 
-    whop = whobyreq.lookup(&req);
     if (whop == 0) {
         // missed pid who, save stats as pid 0
         valp = counts.lookup_or_try_init(&info, &zero);
@@ -174,13 +194,28 @@ if args.ebpf:
     print(bpf_text)
     exit()
 
+if BPF.kernel_struct_has_field(b'request', b'rq_disk') == 1:
+    bpf_text = bpf_text.replace('__RQ_DISK__', 'rq_disk')
+else:
+    bpf_text = bpf_text.replace('__RQ_DISK__', 'q->disk')
+
+if args.pid is not None:
+    bpf_text = bpf_text.replace('FILTER_PID', 'pid != %d' % args.pid)
+else:
+    bpf_text = bpf_text.replace('FILTER_PID', '0')
+
 b = BPF(text=bpf_text)
-b.attach_kprobe(event="blk_account_io_start", fn_name="trace_pid_start")
+if BPF.get_kprobe_functions(b'__blk_account_io_start'):
+    b.attach_kprobe(event="__blk_account_io_start", fn_name="trace_pid_start")
+else:
+    b.attach_kprobe(event="blk_account_io_start", fn_name="trace_pid_start")
 if BPF.get_kprobe_functions(b'blk_start_request'):
     b.attach_kprobe(event="blk_start_request", fn_name="trace_req_start")
 b.attach_kprobe(event="blk_mq_start_request", fn_name="trace_req_start")
-b.attach_kprobe(event="blk_account_io_done",
-    fn_name="trace_req_completion")
+if BPF.get_kprobe_functions(b'__blk_account_io_done'):
+    b.attach_kprobe(event="__blk_account_io_done", fn_name="trace_req_completion")
+else:
+    b.attach_kprobe(event="blk_account_io_done", fn_name="trace_req_completion")
 
 print('Tracing... Output every %d secs. Hit Ctrl-C to end' % interval)
 
@@ -206,7 +241,7 @@ while 1:
         print()
     with open(loadavg) as stats:
         print("%-8s loadavg: %s" % (strftime("%H:%M:%S"), stats.read()))
-    print("%-6s %-16s %1s %-3s %-3s %-8s %5s %7s %6s" % ("PID", "COMM",
+    print("%-7s %-16s %1s %-3s %-3s %-8s %5s %7s %6s" % ("PID", "COMM",
         "D", "MAJ", "MIN", "DISK", "I/O", "Kbytes", "AVGms"))
 
     # by-PID output
@@ -224,7 +259,7 @@ while 1:
 
         # print line
         avg_ms = (float(v.us) / 1000) / v.io
-        print("%-6d %-16s %1s %-3d %-3d %-8s %5s %7s %6.2f" % (k.pid,
+        print("%-7d %-16s %1s %-3d %-3d %-8s %5s %7s %6.2f" % (k.pid,
             k.name.decode('utf-8', 'replace'), "W" if k.rwflag else "R",
             k.major, k.minor, diskname, v.io, v.bytes / 1024, avg_ms))
 
